﻿Imports System.IO
Imports System.Text
Imports System.Runtime.InteropServices
Imports System.Threading

Public Class frmHSI88
    ''=====================================================================================================
    ''Konstanten für die Verbindung zum HSI-88-USB
    ''-----------------------------------------------------------------------------------------------------
    Public Const GENERIC_READ = &H80000000
    Public Const GENERIC_WRITE = &H40000000
    Public Const OPEN_EXISTING = 3
    Public Const FILE_SHARE_READ = &H1
    Public Const FILE_SHARE_WRITE = &H2
    Public Const FILE_FLAG_OVERLAPPED = &H40000000
    Public Const FILE_ATTRIBUTE_NORMAL = &H80
    ''=====================================================================================================


    ''=====================================================================================================
    ''API-Funktionen für die Verbindung zum HSI-88-USB
    ''-----------------------------------------------------------------------------------------------------
    Public Declare Auto Function CreateFile Lib "kernel32.dll" (ByVal lpFileName As String, ByVal dwDesiredAccess As Integer, ByVal dwShareMode As Integer, ByVal lpSecurityAttributes As IntPtr, ByVal dwCreationDisposition As Integer, ByVal dwFlagsAndAttributes As Integer, ByVal hTemplateFile As IntPtr) As Microsoft.Win32.SafeHandles.SafeFileHandle
    Public Declare Auto Function CloseHandle Lib "kernel32" (ByVal hFile As Microsoft.Win32.SafeHandles.SafeFileHandle) As Long
    Public Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
    ''=====================================================================================================


    ''=====================================================================================================
    ''Variable für das Debugging
    ''-----------------------------------------------------------------------------------------------------
    Private fHSI_bol_Debugging As Boolean = False            ''Debugging True = eingeschaltet
    ''=====================================================================================================


    ''=====================================================================================================
    ''Variable die Konfiguration der Rückmeldemodule
    ''-----------------------------------------------------------------------------------------------------
    Public fHSI_int_ModulesLeft As Integer = 1             ''Anzahl der angeschlossenen Module links
    Public fHSI_int_ModulesMiddle As Integer = 1            ''Anzahl der angeschlossenen Module Mitte
    Public fHSI_int_ModulesRight As Integer = 1             ''Anzahl der angeschlossenen Module rechts
    ''=====================================================================================================


    ''=====================================================================================================
    ''Variable für die Verbindung zum HSI-88-USB
    ''-----------------------------------------------------------------------------------------------------
    Private mHSI_str_Devicename As String = "\\.\HsiUsb"   ''Name des HSI-88-USB
    Private mHSI_int_Devicenumber As Integer = 1            ''Nummer des HSI-88-USB
    Private mHSI_sh_DeviceHandle As SafeHandle              ''Handle für das HSI-88-USB
    Private mHSI_fs_Filestream As IO.FileStream             ''Datenstream für das HSI-88-USB
    ''=====================================================================================================


    ''=====================================================================================================
    ''Variable für den Verbindungsstatus zum HSI-88-USB
    ''-----------------------------------------------------------------------------------------------------
    Public mHSI_bol_Connectet As Boolean = False            ''Es besteht eine Verbindung
    ''=====================================================================================================


    ''=====================================================================================================
    ''Variable für die Systemkonfiguration
    ''-----------------------------------------------------------------------------------------------------
    Private mHSI_int_ReadWriteBufferSize As Integer = 255   ''Größe des Sende- und Empfangsbuffer
    ''=====================================================================================================


    ''=====================================================================================================
    ''Variable für Empfangs-Thread von dem HSI-88-USB
    ''-----------------------------------------------------------------------------------------------------
    Public mHSI_bol_ThreadReading As Boolean = True         ''Thread soll Daten empfangen (True)
    Private mHSI_thr_Receiving As Thread                    ''Thread welcher Daten vom HSI-88-USB empfängt
    Private Delegate Sub fmM_DProzessMessages(ByVal l_int_AnswerLength As Integer, ByVal l_bytarr_Annswer() As Byte)
    ''=====================================================================================================


    ''=====================================================================================================
    ''Variable für das Fenster
    ''-----------------------------------------------------------------------------------------------------
    Private mHSI_pic_Status() As System.Windows.Forms.PictureBox


    ''=====================================================================================================
    '' Verbindung mit dem HSI-88-USB trennen
    ''=====================================================================================================
    Public Function mHSI_DeviceClose() As Boolean
        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("Verbindung trennen...")
        ''Dem Thrad mitteilen das keine Daten mehr empfangen werden sollen
        mHSI_bol_ThreadReading = False
        On Error Resume Next
        ''Thread abbrechen
        mHSI_thr_Receiving.Abort()
        ''Devicehandle schließen
        CloseHandle(mHSI_sh_DeviceHandle)
        ''Filestream schließen
        mHSI_fs_Filestream.Close()
        mHSI_fs_Filestream = Nothing
        mHSI_sh_DeviceHandle = Nothing
        On Error GoTo 0
        ''Verbindung zum HSI-88-USB als getrennt kennzeichnen
        mHSI_bol_Connectet = False
        ''Verbindungsstatus auswerten und ausgeben
        Call fHSI_ActionConnectionState()
        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("Verbindung getrennt.")
        Me.Close()
    End Function

    ''=====================================================================================================
    '' Verbindung mit dem HSI-88-USB herstellen
    ''=====================================================================================================
    Public Function mHSI_DeviceOpen() As Boolean
        Dim l_bytarr_WriteBuffer(mHSI_int_ReadWriteBufferSize) As Byte  ''Buffer zum schreiben von Befehlen
        Dim l_bytarr_ReadBuffer(mHSI_int_ReadWriteBufferSize) As Byte   ''Buffer zum lesen von l_str_AnswerStringen
        Dim l_int_AnswerLength As Integer = 0                           ''Länge der Antwort in Bytes
        Dim l_str_AnswerString As String = ""                           ''String der Antwort
        Dim l_int_Loop1 As Integer                                      ''Zählvariable
        Dim l_bol_ErrorOccured As Boolean = False                       ''Es trat ein Fehler auf

        ''Überprüfen ob die Verbindung zum HSI-88-USB bereits besteht
        If mHSI_bol_Connectet = True Then
            MsgBox("Eine Verbindung zum HSI-88-USB besteht bereits!")
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Verbindung besteht bereits!")
            ''Sub verlassen
            Exit Function
        End If

        ''Überprüfen ob Rückmeldemodule angegeben wurden
        If fHSI_int_ModulesLeft = 0 And fHSI_int_ModulesMiddle = 0 And fHSI_int_ModulesRight = 0 Then
            ''Verbindung zum HSI-88-USB als getrennt kennzeichnen
            mHSI_bol_Connectet = False
            MsgBox("Es wurden keine Rückmeldemodule eingestellt!")
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Es wurden keine Rückmeldemodule eingestellt!")
            ''Sub verlassen
            Exit Function
        End If

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print(vbCrLf + "Verbindung zum HSI-88-USB herstellen...")
        Try
            ''HSI-88-USB öffnen und sicheren Handle erstellen
            mHSI_sh_DeviceHandle = CreateFile(mHSI_str_Devicename + mHSI_int_Devicenumber.ToString, GENERIC_READ Or GENERIC_WRITE, FILE_SHARE_READ Or FILE_SHARE_WRITE, IntPtr.Zero, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL Or FILE_FLAG_OVERLAPPED, IntPtr.Zero)
        Catch ex As Exception
            ''Debugging
            MsgBox("Die Verbindung zum HSI-88-USB konnte nicht hergestellt werden!")
            If fHSI_bol_Debugging Then Debug.Print("Verbindung zum HSI-88-USB konnte nicht hergestellt werden!")
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
            ''Sub verlassen
            Exit Function
        End Try
        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("Verbindung zum HSI-88-USB hergestellt.")

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print(vbCrLf + "Teste Verbindung zum HSI-88-USB...")
        ''Überprüfen ob das HSI-88-USB geöffnet werden konnte
        If Not mHSI_sh_DeviceHandle.IsInvalid Then
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Verbindung in ordnung.")
        Else
            MsgBox("Die Verbindung zum HSI-88-USB nicht in ordnung!")
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Verbindung nicht in ordnung!")
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
            ''Sub verlassen
            Exit Function
        End If

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print(vbCrLf + "Öffen des Datenstromes...")
        Try
            ''Filestream erzeugen
            mHSI_fs_Filestream = New IO.FileStream(mHSI_sh_DeviceHandle, IO.FileAccess.ReadWrite, 4096, True)
        Catch ex As Exception
            MsgBox("Der Datenstrom vom HSI-88-USB ist nicht in ordnung!")
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Datenstrom konnte nicht geöffnet werden.")
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
            ''Sub verlassen
            Exit Function
        End Try
        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("Datenstrom geöffnet.")

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print(vbCrLf + "Schalte Binär-Modus ein...")
        ''Das HSI-88-USB in den Binär-Modus schalten
        l_bytarr_WriteBuffer = mHSI_String2ByteArray("t0" + Chr(13))
        mHSI_fs_Filestream.Write(l_bytarr_WriteBuffer, 0, 3)
        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("   Befehel gesendet: 't0'...")

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("   Antwort einlesen...")
        l_int_AnswerLength = mHSI_fs_Filestream.Read(l_bytarr_ReadBuffer, 0, mHSI_int_ReadWriteBufferSize)

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("   Antwortlänge: " + l_int_AnswerLength.ToString + " bytes.")
        l_str_AnswerString = ""
        For l_int_Loop1 = 0 To l_int_AnswerLength - 2
            l_str_AnswerString = l_str_AnswerString + Chr(l_bytarr_ReadBuffer(l_int_Loop1))
        Next
        If fHSI_bol_Debugging Then Debug.Print("   Antwort     : " + l_str_AnswerString)

        ''Überprüfen ob der Binärmodus eingeschalten wurde
        If l_str_AnswerString <> "t0" Then
            MsgBox("Der Binär-Modus konnte nicht eingeschalten werden!")
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Der Binär-Modus konnte nicht eingeschalten werden!")
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
            ''Sub verlassen
            Exit Function
        End If
        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("Der Binär-Modus wurde eingeschalten.")


        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print(vbCrLf + "Stelle die Rückmeldemodulanzahl ein...")
        ''Rückmeldemodulanzahl einstellen
        l_bytarr_WriteBuffer = mHSI_String2ByteArray("sxxx" + Chr(13))
        l_bytarr_WriteBuffer(1) = Convert.ToByte(fHSI_int_ModulesLeft)
        l_bytarr_WriteBuffer(2) = Convert.ToByte(fHSI_int_ModulesMiddle)
        l_bytarr_WriteBuffer(3) = Convert.ToByte(fHSI_int_ModulesRight)
        mHSI_fs_Filestream.Write(l_bytarr_WriteBuffer, 0, 5)
        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("   Befehel gesendet: 's" + fHSI_int_ModulesLeft.ToString + fHSI_int_ModulesMiddle.ToString + fHSI_int_ModulesRight.ToString + "'...")

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("   Antwort einlesen...")
        l_int_AnswerLength = mHSI_fs_Filestream.Read(l_bytarr_ReadBuffer, 0, mHSI_int_ReadWriteBufferSize)

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("   Antwortlänge: " + l_int_AnswerLength.ToString + " bytes.")
        l_str_AnswerString = Chr(l_bytarr_ReadBuffer(0))
        For l_int_Loop1 = 1 To l_int_AnswerLength - 2
            l_str_AnswerString = l_str_AnswerString + ";" + Format(l_bytarr_ReadBuffer(l_int_Loop1))
        Next
        If fHSI_bol_Debugging Then Debug.Print("   Antwort     : " + l_str_AnswerString)

        If l_str_AnswerString.Substring(0, 1) = "s" Then
            ''Anzahl der Rückmeldemodule vergleichen
            If l_bytarr_ReadBuffer(1) <> (fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) Then
                MsgBox("Es wurden nur " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule angemeldet, " + Format(fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) + " wurden erwartet!")
                If fHSI_bol_Debugging Then Debug.Print("   Es wurden nur " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule angemeldet, " + Format(fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) + " wurden erwartet!")
                ''Kennzeichnen das ein Fehler aufgetreten ist
                l_bol_ErrorOccured = True
            Else
                If fHSI_bol_Debugging Then Debug.Print("   Es wurden " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule angemeldet.")
            End If
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("   Antwort einlesen...")
            l_int_AnswerLength = mHSI_fs_Filestream.Read(l_bytarr_ReadBuffer, 0, mHSI_int_ReadWriteBufferSize)
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("   Antwortlänge: " + l_int_AnswerLength.ToString + " bytes.")

            l_str_AnswerString = Chr(l_bytarr_ReadBuffer(0))
            For l_int_Loop1 = 1 To l_int_AnswerLength - 2
                l_str_AnswerString = l_str_AnswerString + ";" + Format(l_bytarr_ReadBuffer(l_int_Loop1))
            Next
            If fHSI_bol_Debugging Then Debug.Print("   Antwort     : " + l_str_AnswerString)
        ElseIf l_str_AnswerString.Substring(0, 1) <> "i" Then
            MsgBox("Vom HSI-88-USB wurde eine falsche Antwort zurück gegeben!")
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Es wurde eine Falsche Antwort zurück gegeben!")
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
            ''Sub verlassen
            Exit Function
        End If

        If l_str_AnswerString.Substring(0, 1) = "i" Then
            ''Anzahl der Rückmeldemodule vergleichen
            If l_bytarr_ReadBuffer(1) <> (fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) Then
                MsgBox("Es wurden nur " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule angemeldet, " + Format(fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) + " wurden erwartet!")
                If fHSI_bol_Debugging Then Debug.Print("   Es wurden nur " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule angemeldet, " + Format(fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) + " wurden erwartet!")
                ''Kennzeichnen das ein Fehler aufgetreten ist
                l_bol_ErrorOccured = True
            Else
                If fHSI_bol_Debugging Then Debug.Print("   Es wurden " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule angemeldet.")
            End If
            If l_int_AnswerLength <> (3 + ((fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) * 3)) Then
                MsgBox("Die Zustände für die " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule wurden nicht vollständig zurückgegeben!")
                If fHSI_bol_Debugging Then Debug.Print("   Die Zustände für die " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule wurden nicht vollständig zurückgegeben!")
                ''Kennzeichnen das ein Fehler aufgetreten ist
                l_bol_ErrorOccured = True
            Else
                If fHSI_bol_Debugging Then Debug.Print("   Die Zustände für die " + Format(l_bytarr_ReadBuffer(1)) + " Rückmeldemodule wurden vollständig zurückgegeben!")
                '' Eine empfangene Antwort bearbeiten
                Call fmM_ProzessMessages(l_int_AnswerLength, l_bytarr_ReadBuffer)
            End If
        Else
            MsgBox("Vom HSI-88-USB wurde eine falsche Antwort zurück gegeben!")
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Es wurde eine Falsche Antwort zurück gegeben!")
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
            ''Sub verlassen
            Exit Function
        End If
        ''Überprüfen ob ein Fehler auftrat
        If l_bol_ErrorOccured Then
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
            ''Sub verlassen
            Exit Function
        End If

        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print(vbCrLf + "Starte den automatischen Nachrichtenempfang...")
        Try
            ''Neuen Thread für den Empfang erzeugen und starten
            mHSI_thr_Receiving = New Thread(AddressOf mHSI_Receive)
            mHSI_thr_Receiving.SetApartmentState(ApartmentState.STA)
            mHSI_thr_Receiving.Start()
        Catch ex As Exception
            MsgBox("Der automatischen Nachrichtenempfang konnte nicht gestartet werden!")
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print("Der automatischen Nachrichtenempfang konnte nicht gestartet werden!")
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
            ''Sub verlassen
            Exit Function
        End Try
        ''Debugging
        If fHSI_bol_Debugging Then Debug.Print("Der automatischen Nachrichtenempfang wurde gestartet.")

        ''Verbindung als Hergestellt kennzeichnen
        mHSI_bol_Connectet = True

        ''Verbindungsstatus auswerten und ausgeben
        Call fHSI_ActionConnectionState()

        Return True
    End Function

    ''=====================================================================================================
    '' Daten vom HSI-88-USB empfangen im Thread
    ''=====================================================================================================
    Private Sub mHSI_Receive()
        Dim l_bytarr_ReadBuffer(mHSI_int_ReadWriteBufferSize) As Byte   ''Buffer zum lesen von l_str_AnswerStringen
        Dim l_int_AnswerLength As Integer = 0                           ''Länge der Antwort in Bytes
        Try
            ''Überprüfen ob der Thread Daten empfangen soll
            While mHSI_bol_ThreadReading
                l_int_AnswerLength = mHSI_fs_Filestream.Read(l_bytarr_ReadBuffer, 0, 255)
                If l_int_AnswerLength > 0 Then
                    ''Einen empfangene Antwort bearbeiten
                    Me.Invoke(New fmM_DProzessMessages(AddressOf fmM_ProzessMessages), l_int_AnswerLength, l_bytarr_ReadBuffer)
                End If
            End While
        Catch ex As ThreadAbortException
            ''Debugging
            If fHSI_bol_Debugging Then
                Debug.Print("")
                Debug.Print("Der automatischen Nachrichtenempfang wurde beendet.")
                Debug.Print("")
            End If
        End Try
    End Sub

    ''=====================================================================================================
    '' Eine empfangene Antwort bearbeiten
    ''=====================================================================================================
    Private Sub fmM_ProzessMessages(ByVal l_int_AnswerLength As Integer, ByVal l_bytarr_Annswer() As Byte)
        Dim l_str_AnswerString As String = ""                           ''String der Antwort
        Dim l_int_Loop1 As Integer                                      ''Zählvariable
        Dim l_int_ModulCount As Integer = 0                             ''Anzahl der gemeldeten Rückmeldemodule
        Dim l_int_ModulNumber As Integer = 0                            ''Rückmeldemodul, auf das sich die Nachricht bezieht
        Dim l_bol_MessageEnd As Boolean = True                          ''Die letzte Nachricht wurde komplett eingelesen
        Dim l_bol_ReadingModuleCount As Boolean = False                 ''Die Modulanzahl soll eingelesen werden
        Dim l_int_ReadedModules As Integer = 0                          ''Anzahl der bereits eingelesenen Module
        Dim l_bol_ReadingModul As Boolean = False                       ''Die Modulnummer soll eingelesen werden
        Dim l_bol_ReadingStatus As Boolean = False                      ''Der Status soll eingelesen werden
        Dim l_int_ReadingStatusIdx As Integer = 0                       ''Statusbyte 0=HiByte 1=LoByte
        Dim l_bytarr_ModulState(2) As Byte                              ''Eingelesener Modulstatus

        If l_int_AnswerLength > 0 Then
            ''Debugging
            If fHSI_bol_Debugging Then Debug.Print(vbCrLf + "   Antwortlänge: " + l_int_AnswerLength.ToString + " bytes.")
            l_str_AnswerString = Chr(l_bytarr_Annswer(0))
            For l_int_Loop1 = 1 To l_int_AnswerLength - 1
                l_str_AnswerString = l_str_AnswerString + ";" + Format(l_bytarr_Annswer(l_int_Loop1))
            Next
            If fHSI_bol_Debugging Then Debug.Print("   Antwort     : " + l_str_AnswerString)
            ''Alle Empfangenen Bytes Bearbeiten
            l_bol_MessageEnd = True
            l_bol_ReadingModuleCount = False
            l_bol_ReadingModul = False
            l_bol_ReadingStatus = False
            l_int_ReadingStatusIdx = 0
            l_int_ReadedModules = 0
            For l_int_Loop1 = 0 To l_int_AnswerLength - 1
                ''Überprüfen ob das Zeichen ein "i" ist und ob die letzte Nachricht wurde komplett eingelesen wurde
                If l_bytarr_Annswer(l_int_Loop1) = Asc("i") And l_bol_MessageEnd = True Then
                    ''Zeichen ist ein "i" -> Rückmeldemodulanzahl einlesen starten
                    l_bol_ReadingModuleCount = True
                    l_int_ReadedModules = 0
                    l_bol_MessageEnd = False
                    Continue For
                End If
                ''Überprüfen ob die Rückmeldemodulanzahl eingelesen werden soll
                If l_bol_ReadingModuleCount = True Then
                    ''Rückmeldemodulanzahl einlesen
                    l_int_ModulCount = l_bytarr_Annswer(l_int_Loop1)
                    If fHSI_bol_Debugging Then Debug.Print("   Anzahl der Rückmeldemodule: " + l_int_ModulCount.ToString + vbCrLf)
                    ''Rückmeldemodulanzahl einlesen beenden
                    l_bol_ReadingModuleCount = False
                    ''Rückmeldemodulummer einlesen starten
                    l_bol_ReadingModul = True
                    Continue For
                End If
                ''Überprüfen ob die Rückmeldemodulnummer eingelesen werden soll
                If l_bol_ReadingModul = True Then
                    ''Rückmeldemodulnummer einlesen
                    l_int_ModulNumber = l_bytarr_Annswer(l_int_Loop1)
                    l_int_ReadedModules = l_int_ReadedModules + 1
                    ''Rückmeldemodulummer einlesen beenden
                    l_bol_ReadingModul = False
                    ''Rückmeldestatus einlesen starten
                    l_bol_ReadingStatus = True
                    l_int_ReadingStatusIdx = 0
                    Continue For
                End If
                ''Überprüfen ob der Status eingelesen werden soll
                If l_bol_ReadingStatus = True Then
                    ''Der Status soll eingelesen werden
                    l_bytarr_ModulState(l_int_ReadingStatusIdx) = l_bytarr_Annswer(l_int_Loop1)
                    l_int_ReadingStatusIdx = l_int_ReadingStatusIdx + 1
                    ''Überprüfen ob das einlesen des Status beendet werde soll
                    If l_int_ReadingStatusIdx > 1 Then
                        ''Stati des Rückmeldemodules verarbeiten
                        Call fHSI_ProzessStates(l_int_ModulNumber, l_bytarr_ModulState)
                        ''Ausgabe der Stati
                        If fHSI_bol_Debugging Then Debug.Print("   Modul: " + l_int_ModulNumber.ToString + ", Status: " + l_bytarr_ModulState(0).ToString + ";" + l_bytarr_ModulState(1).ToString)
                        ''Rückmeldestatus einlesen stoppen
                        l_bol_ReadingStatus = False
                        l_int_ReadingStatusIdx = 0
                        ''Überprüfen ob weitere Rückmeldemodule eingelesen werden sollen
                        If l_int_ReadedModules < l_int_ModulCount Then
                            ''Rückmeldemodulummer einlesen starten
                            l_bol_ReadingModul = True
                        End If
                    End If
                    Continue For
                End If
                ''Überprüfen ob auf das Nachrichtenende gewartet wird
                If l_bytarr_Annswer(l_int_Loop1) = 13 And l_bol_MessageEnd = False Then
                    If fHSI_bol_Debugging Then Debug.Print("   Nachricht komplett." + vbCrLf)
                    ''Nachricht wurde komplett eingelesen
                    l_bol_MessageEnd = True
                    Continue For
                End If
            Next
        End If
        ''Überprüfen ob die letzte Nachricht komplett war
        If l_bol_MessageEnd = False Then
            If fHSI_bol_Debugging Then Debug.Print(vbCrLf + "   Die letzte Nachricht war nicht komplett." + vbCrLf)
        End If
    End Sub

    ''=====================================================================================================
    ''Stati des Rückmeldemodules verarbeiten
    ''=====================================================================================================
    Private Sub fHSI_ProzessStates(ByVal l_int_ModulNumber As Integer, ByVal l_bytarr_ModulState() As Byte)
        Dim l_int_Loop1 As Integer = 0
        For l_int_Loop1 = 1 To 8
            '' Rückmeldestatus ausgeben
            Call fHSI_PrintModulState((l_int_Loop1 + (16 * (l_int_ModulNumber - 1))), (l_bytarr_ModulState(1) And 1))
            l_bytarr_ModulState(1) = l_bytarr_ModulState(1) >> 1
        Next
        For l_int_Loop1 = 9 To 16
            '' Rückmeldestatus ausgeben
            Call fHSI_PrintModulState((l_int_Loop1 + (16 * (l_int_ModulNumber - 1))), (l_bytarr_ModulState(0) And 1))
            l_bytarr_ModulState(0) = l_bytarr_ModulState(0) >> 1
        Next
    End Sub

    ''=====================================================================================================
    '' Einen übergebenen Sting in ein Bytearray umwandeln
    ''=====================================================================================================
    Public Function mHSI_String2ByteArray(ByVal str As String) As Byte()
        Dim enc As New System.Text.ASCIIEncoding
        Return enc.GetBytes(str)
    End Function

    ''=====================================================================================================
    '' Das Fenster soll geschlossen werden
    ''=====================================================================================================
    Private Sub frmHSI88_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        ''Überprüfen ob noch eine Verbindung besteht
        If mHSI_bol_Connectet Then
            ''Es besteht noch eine Verbindung -> Schließen abbrechen
            e.Cancel = True
            ''Verbindung mit dem HSI-88-USB trennen
            Call mHSI_DeviceClose()
        End If
    End Sub

    ''=====================================================================================================
    '' Verbindung herstellen
    ''=====================================================================================================
    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Call mHSI_DeviceOpen()
    End Sub

    ''=====================================================================================================
    '' Verbindung beenden
    ''=====================================================================================================
    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        ''Verbindung mit dem HSI-88-USB trennen
        Call mHSI_DeviceClose()
    End Sub

    ''=====================================================================================================
    '' Verbindungsstatus auswerten und ausgeben
    ''=====================================================================================================
    Private Sub fHSI_ActionConnectionState()
        If mHSI_bol_Connectet = True Then
            labStatus2.Text = "Verbunden"
            Button1.Enabled = False
            Button2.Enabled = True
        Else
            labStatus2.Text = "Nicht verbunden"
            Button1.Enabled = True
            Button2.Enabled = False
        End If
    End Sub

    ''=====================================================================================================
    '' Rückmeldestatus ausgeben
    ''=====================================================================================================
    Private Sub fHSI_PrintModulState(ByVal l_int_Address As Integer, ByVal l_int_State As Integer)
        If l_int_State = 0 Then
            mHSI_pic_Status(l_int_Address - 1).BackColor = System.Drawing.Color.Silver
        Else
            mHSI_pic_Status(l_int_Address - 1).BackColor = System.Drawing.Color.Red
        End If
    End Sub

    ''=====================================================================================================
    '' Fenster laden
    ''=====================================================================================================
    Private Sub frmHSI88_Load(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Load
        Dim l_int_Loop1 As Integer = 0  ''Zählvariable
        Dim l_lng_Left As Long = -12    ''Hilfsvariable
        ''Modulanzahl festlegen
        fHSI_int_ModulesLeft = 1    ''Anzahl der angeschlossenen Module links
        fHSI_int_ModulesMiddle = 1  ''Anzahl der angeschlossenen Module Mitte
        fHSI_int_ModulesRight = 1   ''Anzahl der angeschlossenen Module rechts
        ''Rückmeldeobjekte erzeugen
        ReDim mHSI_pic_Status(((fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) * 16))
        For l_int_Loop1 = 0 To ((fHSI_int_ModulesLeft + fHSI_int_ModulesMiddle + fHSI_int_ModulesRight) * 16) - 1
            mHSI_pic_Status(l_int_Loop1) = New System.Windows.Forms.PictureBox
            If l_int_Loop1 <= 15 Then
                grbModul1.Controls.Add(mHSI_pic_Status(l_int_Loop1))
            ElseIf l_int_Loop1 <= 31 Then
                grbModul2.Controls.Add(mHSI_pic_Status(l_int_Loop1))
            Else
                grbModul3.Controls.Add(mHSI_pic_Status(l_int_Loop1))
            End If
            If l_int_Loop1 = 16 Then
                l_lng_Left = -12
            ElseIf l_int_Loop1 = 32 Then
                l_lng_Left = -12
            End If
            l_lng_Left = l_lng_Left + 18
            mHSI_pic_Status(l_int_Loop1).BackColor = System.Drawing.Color.Silver
            mHSI_pic_Status(l_int_Loop1).BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle
            mHSI_pic_Status(l_int_Loop1).Location = New System.Drawing.Point(l_lng_Left, 19)
            mHSI_pic_Status(l_int_Loop1).Name = "picStatus" + l_int_Loop1.ToString
            mHSI_pic_Status(l_int_Loop1).Size = New System.Drawing.Size(12, 12)
            mHSI_pic_Status(l_int_Loop1).TabIndex = l_int_Loop1
            mHSI_pic_Status(l_int_Loop1).TabStop = False
        Next
    End Sub
End Class
